@@ -1,5 +1,8 @@
 #!/usr/bin/python3
 # -*- coding: utf-8 -*-
+"""
+Just the init!
+"""
 
 from .builder import CFGBuilder
 from .model import Block, Link, CFG
