@@ -3,7 +3,6 @@
 """
 This is the wrapper that provides/is the py2cfg shell script.
 """
-# Taylor 2019
 
 import os
 import sys
@@ -17,7 +16,7 @@ sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
 from py2cfg import CFGBuilder
 
 
-def main():
+if __name__ == "__main__":
     parser = argparse.ArgumentParser(
         description="Generate the control flow graph of a Python program"
     )
@@ -25,6 +24,23 @@ def main():
         "input_file",
         help="Path to a file containing a Python program for which the CFG must be generated",
     )
+    parser.add_argument(
+        "--format",
+        nargs=1,
+        type=str,
+        default="svg",
+        help="File format of the generated cfg",
+    )
+    parser.add_argument(
+        "--calls",
+        nargs=1,
+        type=bool,
+        default=True,
+        help="Toggle call graph",
+    )
+    parser.add_argument(
+        "--show", nargs=1, type=bool, default=False, help="Open CFG after generation"
+    )
 
     args = parser.parse_args()
     cfg_name = args.input_file.split("/")[-1]
@@ -32,12 +48,14 @@ def main():
 
     # Some options for wrapping:
     # cfg.build_visual(cfg_name[:-3] + '_cfg', format='pdf', calls=True)
-    cfg.build_visual(cfg_name[:-3] + "_cfg", format="png", calls=True, show=False)
+    # cfg.build_visual(cfg_name[:-3] + "_cfg", format="svg", calls=True, show=False)
+    # cfg.build_visual(cfg_name[:-3] + "_cfg", format="png", calls=True, show=False)
     # cfg.build_visual('controlflowgraph', format='png', calls=True, show=False)
-
+    cfg.build_visual(
+        cfg_name[:-3] + "_cfg",
+        format=args.format,
+        calls=args.calls,
+        show=args.show,
+    )
     # removes the CFG file, which maybe we could just turn off?
     os.remove(cfg_name[:-3] + "_cfg")
-
-
-if __name__ == "__main__":
-    main()
