@@ -1,6 +1,5 @@
 #!/usr/bin/python3
 # -*- coding: utf-8 -*-
-
 """
 Control flow graph for Python programs.
 """
@@ -8,7 +7,6 @@ Control flow graph for Python programs.
 
 from typing import List, Optional, Iterator
 import ast  # type: ignore
-from _ast import Compare  # type: ignore
 import astor  # type: ignore
 import graphviz as gv  # type: ignore
 from graphviz.dot import Digraph  # type: ignore
@@ -23,6 +21,7 @@ class Block(object):
     a list of Links that represent control flow jumps.
     """
 
+    # Look into ctypes.Structure
     __slots__ = ["id", "statements", "func_calls", "predecessors", "exits"]
 
     def __init__(self, id: int) -> None:
@@ -102,6 +101,10 @@ class Block(object):
             txt += func_name + "\n"
         return txt
 
+    def type(self, default=None):
+        default = ast.Pass if default is None else default
+        return type(self.statements[0]) if self.statements else default
+
 
 class Link(object):
     """
@@ -115,10 +118,13 @@ class Link(object):
     __slots__ = ["source", "target", "exitcase"]
 
     def __init__(
-        self, source: Block, target: Block, exitcase: Optional[Compare] = None
+        self,
+        source: Block,
+        target: Block,
+        exitcase: Optional[ast.Compare] = None,
     ) -> None:
-        assert type(source) == Block, "Source of a link must be a block"
-        assert type(target) == Block, "Target of a link must be a block"
+        assert isinstance(target, Block), "Source of a link must be a block"
+        assert isinstance(target, Block), "Target of a link must be a block"
         # Block from which the control flow jump was made.
         self.source = source
         # Target block of the control flow jump.
@@ -127,11 +133,13 @@ class Link(object):
         self.exitcase = exitcase
 
     def __str__(self) -> str:
-        return "link from {} to {}".format(str(self.source), str(self.target))
+        return f"link from {self.source} to {self.target}"
 
     def __repr__(self) -> str:
+        # This isn't how repr is supposed to be used... We should be able to
+        # deep copy this object by calling eval(repr(link))`.
         if self.exitcase is not None:
-            return "{}, with exitcase {}".format(str(self), ast.dump(self.exitcase))
+            return f"{self}, with exitcase {ast.dump(self.exitcase)}"
         return str(self)
 
     def get_exitcase(self) -> str:
@@ -157,9 +165,21 @@ class CFG(object):
     CFG).
     """
 
+    # Also serves as graph Key table
+    node_styles = {
+        "input": ("parallelogram", "#afeeee"),  # Pale Turquoise
+        "default": ("rectangle", "#FFFB81"),  # Pale Yellow
+        ast.If: ("diamond", "#FF6752"),  # Pale Red
+        ast.For: ("hexagon", "#FFBE52"),  # Pale Orange
+        ast.While: ("hexagon", "#FFBE52"),  # Pale Orange
+        ast.Call: ("tab", "#E552FF"),  # Pale Purple
+        ast.Return: ("parallelogram", "#98fb98"),  # Pale Green
+    }
+    DEFAULT = node_styles["default"]
+
     def __init__(self, name: str, asynchr: bool = False) -> None:
-        assert type(name) == str, "Name of a CFG must be a string"
-        assert type(asynchr) == bool, "Async must be a boolean value"
+        assert isinstance(name, str), "Name of a CFG must be a string"
+        assert isinstance(asynchr, bool), "Async must be a boolean value"
         # Name of the function or module being represented.
         self.name = name
         # Type of function represented by the CFG (sync or async). A Python
@@ -173,22 +193,6 @@ class CFG(object):
         self.functioncfgs = {}
         # Sub-CFGs
         self.classcfgs = {}
-        self.shapes = {
-            "if": "diamond",
-            "loop": "hexagon",
-            "calls": "tab",
-            "return": "parallelogram",
-            "default": "rectangle",
-            "input": "parallelogram",
-        }
-        self.colors = {
-            "if": "#FF6752",  # Pale Red
-            "loop": "#FFBE52",  # Pale Orange
-            "calls": "#E552FF",  # Pale Purple
-            "return": "#98fb98",  # Pale Green
-            "input": "#afeeee",  # Pale Turquoise
-            "default": "#FFFB81",
-        }  # Pale Yellow
 
     def __str__(self) -> str:
         return "CFG for {}".format(self.name)
@@ -198,103 +202,105 @@ class CFG(object):
         key_subgraph = gv.Digraph(
             name="cluster_KEY", format=format, graph_attr={"label": "KEY"}
         )
-        key_subgraph.node(
-            name="if",
-            _attributes={
-                "shape": self.shapes["if"],
-                "style": "filled",
-                "fillcolor": self.colors["if"],
-            },
-        )
-        key_subgraph.node(
-            name="loop",
-            _attributes={
-                "shape": self.shapes["loop"],
-                "style": "filled",
-                "fillcolor": self.colors["loop"],
-            },
-        )
-        key_subgraph.node(
-            name="calls",
-            _attributes={
-                "shape": self.shapes["calls"],
-                "style": "filled",
-                "fillcolor": self.colors["calls"],
-            },
-        )
-        key_subgraph.node(
-            name="return",
-            _attributes={
-                "shape": self.shapes["return"],
-                "style": "filled",
-                "fillcolor": self.colors["return"],
-            },
-        )
-        key_subgraph.node(
-            name="default",
-            _attributes={
-                "shape": self.shapes["default"],
-                "style": "filled",
-                "fillcolor": self.colors["default"],
-            },
-        )
-        key_subgraph.node(
-            name="input",
-            _attributes={
-                "shape": self.shapes["input"],
-                "style": "filled",
-                "fillcolor": self.colors["input"],
-            },
-        )
+        for typeobj, (shape, color) in zip(
+            self.node_styles.keys(), self.node_styles.values()
+        ):
+            key_subgraph.node(
+                name=(
+                    typeobj.__name__.lower()
+                    if isinstance(typeobj, type)
+                    else str(typeobj)
+                ),
+                _attributes={
+                    "shape": shape,
+                    "style": "filled",
+                    "fillcolor": color,
+                },
+            )
+
+        # Adding edge helps with alignment.
         key_subgraph.edge("if", "input", _attributes={"style": "invis"})
-        key_subgraph.edge("input", "calls", _attributes={"style": "invis"})
-        key_subgraph.edge("loop", "return", _attributes={"style": "invis"})
+        key_subgraph.edge("input", "call", _attributes={"style": "invis"})
+        key_subgraph.edge("for", "return", _attributes={"style": "invis"})
         key_subgraph.edge("return", "default", _attributes={"style": "invis"})
         return key_subgraph
 
+    # default handler for graph nodes
+    def _style_handler(self, block, typeobject):
+
+        shape, color = self.node_styles.get(typeobject, self.DEFAULT)
+        # Left-align all remaining blocks
+        text = "\l".join(line for line in block.get_source().splitlines())
+        return shape, color, text
+
+    # default handler for graph edges
+    def _edge_handler(self, link):
+        return None, "black", link.get_exitcase().strip()
+
+    def stylize_node(self, block: Block, default=None) -> tuple:
+        # Since we're modelling ast objects, we might as well copy how they
+        # implement handlers. This should allow us to implement new node types
+        # simply defining a method `node_ASTNAME`
+        typeobject = block.type(default)
+        name = typeobject.__name__ if isinstance(typeobject, type) else typeobject
+        style = getattr(
+            self,
+            f"node_{name}",
+            self._style_handler,
+        )(block, typeobject)
+        assert len(style) == 3, "Style handler must return (shape, color, label)"
+        assert all(
+            isinstance(attr, str) or attr or attr is None for attr in style
+        ), "Style attributes must be str or NoneType"
+        return style
+
+    def stylize_edge(
+        self,
+        link: Link,
+        default=None,
+        default_target=None,
+    ) -> tuple:
+        typeobject = link.source.type(default)
+        name = typeobject.__name__ if isinstance(typeobject, type) else typeobject
+        target_type = link.target.type(default_target)
+        target_name = (
+            target_type.__name__ if isinstance(typeobject, type) else typeobject
+        )
+        # Do specific attribute search of source->dest
+        style_fn = getattr(self, f"{name}_to_{target_name}", None)
+
+        # Do generic attribute search of source->any
+        if style_fn is None:
+            style_fn = getattr(self, f"edge_{name}", self._edge_handler)
+        style = style_fn(link)
+        assert len(style) == 3, "style handler must return (shape, color, label)"
+        assert all(
+            isinstance(attr, str) or attr is None for attr in style
+        ), "style attributes must be str or NoneType"
+        return style
+
     def _visit_blocks(
-        self, graph: Digraph, block: Block, visited: List[int] = [], calls: bool = True
+        self,
+        graph: Digraph,
+        block: Block,
+        visited: List[int] = [],
+        calls: bool = True,
     ) -> None:
         # Don't visit blocks twice.
         if block.id in visited:
             return
-
-        nodelabel = block.get_source()  # label = nodelabel
-        # Use the first statement to find out what's in the node
-        # Just convert it to a string for simplicity
-        # This saves us from parsing the entire label and having to handle strings
-        # or even variable names that contain keywords
-        node_type = str(type(block.statements[0]))
-
-        nodeshape = self.shapes["default"]
-        nodecolor = self.colors["default"]
-        # _ast.Expr is the generic case (think base-level AST node),
-        # _ast.Assign is also unused here in case we want to highlight them differently
-        if "_ast.ClassDef" in node_type:
-            nodelabel = nodelabel.splitlines()[0] + " ...\n"
-        if "_ast.If" in node_type:
-            nodeshape = self.shapes["if"]
-            nodecolor = self.colors["if"]
-        elif ("_ast.For" in node_type) or ("_ast.While" in node_type):
-            nodeshape = self.shapes["loop"]
-            nodecolor = self.colors["loop"]
-        elif "_ast.Return" in node_type:
-            nodeshape = self.shapes["return"]
-            nodecolor = self.colors["return"]
-        else:
-            # Left-align all remaining blocks
-            tmp = ""
-            for line in nodelabel.splitlines():
-                line += "\l"
-                tmp += line
-            nodelabel = tmp
-
+        visited.append(block.id)
+        nodeshape, nodecolor, nodelabel = self.stylize_node(block)
+        node_type = block.type()
         graph.node(
             str(block.id),
             label=nodelabel,
-            _attributes={"style": "filled", "shape": nodeshape, "fillcolor": nodecolor},
+            _attributes={
+                "style": "filled",
+                "shape": nodeshape,
+                "fillcolor": nodecolor,
+            },
         )
-        visited.append(block.id)
 
         # Show the block's function calls in a node.
         # TODO(jstuder) Include conditional blocks in "calls" edges
@@ -305,8 +311,7 @@ class CFG(object):
             tmp = ""
             for line in calls_label.splitlines():
                 if "input" in line:
-                    nodeshape = self.shapes["input"]
-                    nodecolor = self.colors["input"]
+                    nodeshape, nodecolor = self.node_styles["input"]
                     graph.node(
                         str(block.id) + "_input",
                         label=line,
@@ -322,13 +327,14 @@ class CFG(object):
                     line += "\l"
                     tmp += line
             calls_label = tmp
+            shape, color = self.node_styles[ast.Call]
             graph.node(
                 calls_node,
                 label=calls_label,
                 _attributes={
-                    "shape": self.shapes["calls"],
+                    "shape": shape,
                     "style": "filled",
-                    "fillcolor": self.colors["calls"],
+                    "fillcolor": color,
                 },
             )
 
@@ -341,20 +347,9 @@ class CFG(object):
 
         # Recursively visit all the blocks of the CFG.
         for exit in block.exits:
-            edgecolor = "black"
+            assert block == exit.source
             self._visit_blocks(graph, exit.target, visited, calls=calls)
-            edgelabel = exit.get_exitcase().strip()
-            if (
-                ("_ast.If" in node_type)
-                or ("_ast.For" in node_type)
-                or ("_ast.While" in node_type)
-            ):
-                edgecolor = "red"
-                if (edgelabel in nodelabel) and (len(edgelabel) > 0):
-                    edgecolor = "green"
-            if "_ast.ClassDef" in node_type:
-                insert_edge = False
-
+            edgeshape, edgecolor, edgelabel = self.stylize_edge(exit)
             graph.edge(
                 str(block.id),
                 str(exit.target.id),
@@ -372,7 +367,10 @@ class CFG(object):
         # Build the subgraphs for the function definitions in the CFG and add
         # them to the graph.
         for subcfg in self.classcfgs:
-            subgraph = self.classcfgs[subcfg]._build_visual(format=format, calls=calls)
+            subgraph = self.classcfgs[subcfg]._build_visual(
+                format=format,
+                calls=calls,
+            )
             graph.subgraph(subgraph)
 
         for subcfg in self.functioncfgs:
@@ -423,3 +421,31 @@ class CFG(object):
 
         for subcfg in self.functioncfgs.values():
             yield from subcfg
+
+    def node_ClassDef(self, block: Block, typeobj: type):
+        text = block.get_source().splitlines()[0] + "...\n"
+        return *self.node_styles.get(typeobj, self.DEFAULT), text
+
+    def edge_If(self, link):
+        _, _, nodelabel = self.stylize_node(link.source)
+        color = "red"
+        edgelabel = link.get_exitcase().strip()
+        if edgelabel in nodelabel:
+            color = "green"
+        return None, color, edgelabel
+
+    def edge_While(self, link):
+        _, _, nodelabel = self.stylize_node(link.source)
+        color = "red"
+        edgelabel = link.get_exitcase().strip()
+        if edgelabel in nodelabel:
+            color = "green"
+        return None, color, edgelabel
+
+    def edge_For(self, link):
+        _, _, nodelabel = self.stylize_node(link.source)
+        color = "red"
+        edgelabel = link.get_exitcase().strip()
+        if edgelabel in nodelabel:
+            color = "green"
+        return None, color, edgelabel
